use crate::hand_view::HandView;
use crate::tile_view::PlacedTileView;
use board_shared::board::Board;
use board_shared::expr::is_valid_guess;
use board_shared::game::Game;
use board_shared::tile::Tile;
use gloo_dialogs::alert;
use yew::prelude::*;

enum SelectedTile {
    InHand(usize),
    Equals,
    None,
}

#[derive(Properties, PartialEq)]
struct BoardViewProps {
    board: Board,
    on_click: Callback<(usize, usize)>,
}

#[function_component(BoardView)]
fn board_view(BoardViewProps { board, on_click }: &BoardViewProps) -> Html {
    html! {
        <table class="board">
            { (0..25).map(|x| html! {
                <tr class="board-row">
                { (0..25).map(|y| html! {
                    <PlacedTileView x={x} y={y} key={x} tile={board.get(x, y)} on_click={on_click.clone()} />
                }).collect::<Html>() }
                </tr>
            }).collect::<Html>() }
        </table>
    }
}

#[function_component(App)]
pub fn app() -> Html {
    let game = use_state(Game::default);
    let current_game = use_state(Game::default);

    let selected_tile = use_state(|| SelectedTile::None);
    let on_tile_click = {
        let game = current_game.clone();
        let selected_tile = selected_tile.clone();
        Callback::from(move |(x, y)| {
            if let SelectedTile::InHand(idx) = *selected_tile {
                let mut in_hand = game.in_hand.clone();
                let tile = in_hand.tiles.remove(idx);
                let mut board = game.board.clone();
                board.set(x, y, tile);
                game.set(Game { board, in_hand });
                selected_tile.set(SelectedTile::None);
            } else if let SelectedTile::Equals = *selected_tile {
                let mut board = game.board.clone();
                board.set(x, y, Tile::Equals);
                game.set(Game {
                    board,
                    in_hand: game.in_hand.clone(),
                });
                selected_tile.set(SelectedTile::None);
            }
        })
    };
    let on_tile_select = {
        let selected_tile = selected_tile.clone();
        Callback::from(move |idx| {
            selected_tile.set(SelectedTile::InHand(idx));
        })
    };
    let on_equals_select = {
        Callback::from(move |_| {
            selected_tile.set(SelectedTile::Equals);
        })
    };
    let on_continue_click = {
        let current_game = current_game.clone();
        Callback::from(move |_| {
            let diff = game.board.difference(&current_game.board);
            if let Some(true) = Board::is_contiguous(&diff) {
                if let Ok(true) = is_valid_guess(&current_game.board, &diff) {
                    alert("Valid move!");
                    let mut in_hand = current_game.in_hand.clone();
                    in_hand.complete();
                    game.set(Game {
                        board: current_game.board.clone(),
                        in_hand: in_hand.clone()
                    });
                    current_game.set(Game {
                        board: current_game.board.clone(),
                        in_hand,
                    });
                } else {
                    alert("Invalid move! (invalid expressions)");
                    current_game.set(Game {
                        board: game.board.clone(),
                        in_hand: game.in_hand.clone(),
                    });
                }
            } else {
                if !diff.is_empty() {
                    alert("Invalid move! (not contiguous)");
                }
                let mut in_hand = game.in_hand.clone();
                in_hand.complete();
                current_game.set(Game {
                    board: game.board.clone(),
                    in_hand,
                });
            }
        })
    };
    html! {
        <main>
            <BoardView board={current_game.board.clone()} on_click={on_tile_click} />
            <HandView hand={current_game.in_hand.clone()} on_select={on_tile_select} />
            <div class="row">
                <button onclick={on_equals_select} class="button">{"="}</button>
                <button onclick={on_continue_click} class="button">{if current_game.in_hand.tiles.is_empty() { "Start" } else { "Continue" }}</button>
            </div>
        </main>
    }
}
